@@ -1,5 +1,5 @@
 /*
- * hydra v1.7 (c) 2001-2002 by van Hauser / THC <vh@reptile.rug.ac.be>
+ * hydra v2.4 (c) 2001-2003 by van Hauser / THC <vh@thc.org>
  * http://www.hackerschoice.com
  *
  * Parallized network login hacker. Do only use for legal purposes.
@@ -26,18 +26,19 @@ extern void service_http(unsigned long int ip, int sp, unsigned char options, ch
 extern void service_icq(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE *fp, int port);
 extern void service_pcnfs(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE *fp, int port);
 extern void service_smb(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE *fp, int port);
+extern void service_mysql(unsigned long int ip, int sp, unsigned char options, char *miscptr, FILE *fp, int port);
 // ADD NEW SERVICES HERE
-#define SERVICES  "telnet ftp pop3 imap  http https smb cisco cisco-enable ldap nntp vnc rexec socks5 icq pcnfs"
+#define SERVICES  "telnet ftp pop3 imap smb http https cisco cisco-enable ldap mysql nntp vnc rexec socks5 icq pcnfs"
 // ADD NEW SERVICES HERE
 
 #define MAXBUF    264
 #define MAXTASKS  255
 
 #define PROGRAM   "Hydra"
-#define VERSION   "v2.1"
+#define VERSION   "v2.4"
 #define AUTHOR    "van Hauser / THC"
-#define EMAIL     "<vh@reptile.rug.ac.be>"
-#define RESSOURCE "http://www.thehackerschoice.com"
+#define EMAIL     "<vh@thc.org>"
+#define RESSOURCE "http://www.thc.org"
 
 #define DEBUG	  0
 #define WAITTIME  30
@@ -55,33 +56,42 @@ int killed = 0;
 static pid_t pids[MAXTASKS];
 
 void help() {
-    printf("%s %s (c) 2002 by %s %s
-Syntax:  %s [[[-l LOGIN|-L FILE] [-p PASS|-P FILE]] | [-C FILE]] [-o FILE] [-t TASKS] [-g TASKS] [-w TIME] [-f] [-e ns] [-s PORT] [-S] server service [OPT]\n",
-PROGRAM,VERSION,AUTHOR,EMAIL,prg);
-printf("\nOptions:");
-#ifdef HYDRA_SSL
-printf("
-    -S        connect via SSL");
+    printf("%s %s (c) 2003 by %s %s " \
+           "Syntax:  %s [[[-l LOGIN|-L FILE] [-p PASS|-P FILE]] | " \
+           "[-C FILE]] [-o FILE] [-t TASKS] [-g TASKS] [-w TIME] " \
+           "[-f] [-e ns] [-s PORT] [-S] server service [OPT]\n", \
+           PROGRAM,VERSION,AUTHOR,EMAIL,prg);
+    printf("\nOptions:\n");
+#ifdef OPENSSL
+    printf("  -S        connect via SSL\n");
 #endif
-printf("
-    -l LOGIN or -L FILE login with LOGIN name, or load several logins from FILE
-    -p PASS  or -P FILE try password PASS, or load several passwords from FILE
-    -e ns     additional checks, \"n\" for null password, \"s\" try login as pass
-    -C FILE   colon seperated \"login:pass\" format, instead of -L/-P option
-    -o FILE   write found login/password pairs to FILE instead of stdout
-    -f        exit after the first found login/password pair
-    -t TASKS  run TASKS number of connects in parallel (default: %d)
-    -g TASKS  start TASKS number per second until -t TASKS are reached
-    -w TIME   in seconds, defines the max wait reply time (default: %d)
-    -s PORT   if the service is on a different default port, define it here
-    server    the target server
-    service   the service to crack. Supported protocols: [%s]
-    OPT       some service modules need additional input, put it here
-
-%s is a tool to guess valid login/password pairs on a target server.
-You can always find the newest version at %s
-Use this tool only for legal purposes!
-",TASKS,WAITTIME,SERVICES,PROGRAM,RESSOURCE);
+    printf("  -s PORT   if the service is on a different default port, " \
+             "define it here\n" \
+           "  -l LOGIN  or -L FILE login with LOGIN name, or load " \
+             "several logins from FILE\n" \
+           "  -p PASS   or -P FILE try password PASS, or load several " \
+             "passwords from FILE\n" \
+           "  -e ns     additional checks, \"n\" for null password, \"s\" " \
+             "try login as pass\n" \
+           "  -C FILE   colon seperated \"login:pass\" format, instead of " \
+             "-L/-P option\n" \
+           "  -o FILE   write found login/password pairs to FILE instead " \
+             "of stdout\n" \
+           "  -f        exit after the first found login/password pair\n" \
+           "  -t TASKS  run TASKS number of connects in parallel " \
+             "(default: %d)\n" \
+           "  -g TASKS  start TASKS number per second until -t TASKS are" \
+             "reached\n" \
+           "  -w TIME   in seconds, defines the max wait reply time " \
+             "(default: %d)\n" \
+           "  server    the target server\n" \
+           "  service   the service to crack. Supported protocols: [%s]\n" \
+           "  OPT       some service modules need additional input, put " \
+              "it here\n" \
+           "\n%s is a tool to guess valid login/password pairs on a "\
+           "target server.\nYou can always find the newest version at %s\n" \
+           "Use this tool only for legal purposes!\n", \
+           TASKS,WAITTIME,SERVICES,PROGRAM,RESSOURCE);
     exit(-1);
 }
 
@@ -208,7 +218,7 @@ void fill_mem(char *ptr, FILE *fp) {
             case 'w': waittime = atoi(optarg); break;
             case 's': port = atoi(optarg); break;
             case 'S':
-#ifndef HYDRA_SSL
+#ifndef OPENSSL
 		      fprintf(stderr, "Sorry, hydra was compiled without SSL support. Install openssl and recompile!\n");
                       ssl = 0; break;
 #else 
@@ -231,6 +241,7 @@ void fill_mem(char *ptr, FILE *fp) {
     if (strcmp(service, "nntp") == 0) i = 1;
     if (strcmp(service, "socks5") == 0) i = 1;
     if (strcmp(service, "icq") == 0) i = 1;
+    if (strcmp(service, "mysql") == 0) i = 1;
 // ADD NEW SERVICES HERE
     if (strcmp(service, "smb") == 0) {
         if (tasks > 1) {
@@ -284,7 +295,7 @@ void fill_mem(char *ptr, FILE *fp) {
         if (*miscptr != '/') bail("The web page you supplied must start with a \"/\", e.g. \"/protected/login\"");
         strcpy(service, "www");
     }
-#ifdef HYDRA_SSL
+#ifdef OPENSSL
     if (strcmp(service, "ssl") == 0 || strcmp(service, "https") == 0) {
         i = 1; 
         if (miscptr == NULL) bail("You must supply the web page as an additional option or via -m");
@@ -371,7 +382,7 @@ void fill_mem(char *ptr, FILE *fp) {
     if (ssl)
         options = options | OPTION_SSL;
 
-    printf("%s %s (c) 2002 by %s - use allowed only for legal purposes.\n", PROGRAM, VERSION, AUTHOR);
+    printf("%s %s (c) 2003 by %s - use allowed only for legal purposes.\n", PROGRAM, VERSION, AUTHOR);
     printf("%s is starting! [parallel tasks: %d, login tries: %lu (l:%d/p:%d)]\n",PROGRAM,tasks,todo,countlogin,countpass);
 
     math2 = math2 / tasks;
@@ -418,10 +429,15 @@ void fill_mem(char *ptr, FILE *fp) {
     
 #ifdef NESSUS_PLUGIN    
     svc_kb_name = malloc(40 + strlen(service));
+    if(strcmp(service, "http") == 0)
+      strcpy(svc_kb_name, "Services/www");
+    else
     sprintf(svc_kb_name, "Services/%s", service);
     asc_port = plug_get_key(nessus, svc_kb_name);
     if (asc_port)port = atoi(asc_port);
     free(svc_kb_name);
+    if(port && host_get_port_state(nessus, port) == 0)
+    	return 0;
     if (port && IS_ENCAPS_SSL(plug_get_port_transport(nessus, port)))
 		options |= OPTION_SSL;
     if (soc >= 0)
@@ -456,6 +472,7 @@ void fill_mem(char *ptr, FILE *fp) {
     if (strcmp(service, "icq") == 0) service_icq(ip, socketpairs[i][1], options, miscptr, ofp, port);
     if (strcmp(service, "pcnfs") == 0) service_pcnfs(ip, socketpairs[i][1], options, miscptr, ofp, port);
     if (strcmp(service, "smb") == 0) service_smb(ip, socketpairs[i][1], options, miscptr, ofp, port);
+    if (strcmp(service, "mysql") == 0) service_mysql(ip, socketpairs[i][1], options, miscptr, ofp, port);
 // ADD NEW SERVICES HERE
                 return 0;
             }
@@ -522,29 +539,40 @@ void fill_mem(char *ptr, FILE *fp) {
                                     if (rc == 'N' || rc == 'F') {
                                         done = 1;
                                         sent++;
-                                        if (verbose) {
-                                            if (b < try_null_password + try_password_same_as_login) {
-                                                if (try_null_password && b == 0)
-                                                    printf("New pair: login \"%s\" - pass \"\" \t(%lu of %lu completed)\n", login_ptr, sent, todo);
-                                                if (try_password_same_as_login && ((b == 0 && try_null_password == 0) || (b == 1 && try_null_password)))
-                                                    printf("New pair: login \"%s\" - pass \"%s\" \t(%lu of %lu completed)\n", login_ptr, login_ptr, sent, todo);
-                                            } else
-                                                printf("New pair: login \"%s\" - pass \"%s\" \t(%lu of %lu completed)\n", login_ptr, pass_ptr, sent, todo);
-                                        }
                                         memset(&sb, 0, sizeof(sb));
                                         length = strlen(login_ptr) + 1;
                                         strcpy(sb, login_ptr);
-                                        if (try_password_same_as_login + try_null_password < b) {
-                                            if (try_null_password && b == 0)
+                                        if (b < try_password_same_as_login + try_null_password) {
+                                            if (try_null_password && b == 0) {
                                                 length += 1;
+                                            }
                                             if (try_password_same_as_login && ((b == 0 && try_null_password == 0) || (b == 1 && try_null_password))) {
                                                 strcpy(sb + length, login_ptr);
                                                 length += strlen(login_ptr) + 1;
                                             }
                                         } else {
-                                            strcpy(sb + length, pass_ptr);
+                                            while (((try_password_same_as_login && strcmp(pass_ptr, login_ptr) == 0)
+                                                   ||
+                                                  (try_null_password && strlen(pass_ptr) == 0)) && b < countpass) {
+                                                if (verbose)
+                                                    printf("Detected double with -e n|s option, skipping double password try. %s <-> %s\n", login_ptr, pass_ptr);
+                                                pass_ptr += strlen(pass_ptr) + 1;
+                                                b++;
+                                                sent++;
+                                            }
+                                            if (b == countpass) {
+                                                strcpy(sb + length, "THChydra");
+                                                length += 9;
+                                                b = countpass - 1;
+                                                if (sent > todo)
+                                                    sent = todo;
+                                            } else {
+                                                strcpy(sb + length, pass_ptr);
+                                                length += strlen(pass_ptr) + 1;
+                                            }
                                         }
-                                        length += strlen(pass_ptr) + 1;
+                                        if (verbose)
+                                            printf("New pair: login \"%s\" - pass \"%s\" \t(%lu of %lu completed)\n", sb, sb + strlen(sb) + 1, sent, todo);
                                         write(socketpairs[c][0], sb, length);
                                         if (debug) printf("Pair sent to process %d\n", pids[c]);
                                     } else {
